# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function, unicode_literals
from . import config
import os
import re
import gettext
from .logging import debugLogger as logger

"""
Internationalization facilities
"""
__author__ = """David Malcolm <dmalcolm@redhat.com>, Zack Cerza <zcerza@redhat.com>"""


"""
Singleton list of TranslationDb instances, to be initialized by the script with
whatever translation databases it wants.
"""
translationDbs = []


class TranslationDb(object):
    """
    Abstract base class representing a database of translations
    """
    def getTranslationsOf(self, srcName):
        """
        Pure virtual method to look up the translation of a string.
        Returns a list of candidate strings (the translation), empty if not found.

        Note that a source string can map to multiple translated strings. For
        example, in the French translation of Evolution, the string "Forward" can
        translate to both
        (i) "Faire suivre" for forwarding an email, and
        (ii) "Suivant" for the next page in a wizard.
        """
        raise NotImplementedError


class GettextTranslationDb(TranslationDb):
    """
    Implementation of TranslationDb which leverages gettext, using a single
    translation mo-file.
    """
    def __init__(self, moFile):
        self.__moFile = moFile
        self.__gnutranslations = gettext.GNUTranslations(open(moFile))

    def getTranslationsOf(self, srcName):
        # print "searching for translations of %s"%srcName
        # Use a dict to get uniqueness:
        results = {}
        result = self.__gnutranslations.ugettext(srcName)
        if result != srcName:
            results[result] = None

        # Hack alert:
        #
        # Note that typical UI definition in GTK etc contains strings with
        # underscores to denote accelerators.
        # For example, the stock GTK "Add" item has text "_Add" which e.g.
        # translates to "A_jouter" in French
        #
        # Since these underscores have been stripped out before we see these strings,
        # we are looking for a translation of "Add" into "Ajouter" in this case, so
        # we need to fake it, by looking up the string multiple times, with underscores
        # inserted in all possible positions, stripping underscores out of the result.
        # Ugly, but it works.

        for index in range(len(srcName)):
            candidate = srcName[:index] + "_" + srcName[index:]
            result = self.__gnutranslations.ugettext(candidate)
            if result != candidate:
                # Strip out the underscore, and add to the result:
                results[result.replace('_', '')] = True

        return list(results.keys())


def translate(srcString):
    """
    Look up srcString in the various translation databases (if any), returning
    a list of all matches found (potentially the empty list)
    """
    # Use a dict to get uniqueness:
    results = {}
    # Try to translate the string:
    for translationDb in translationDbs:
        for result in translationDb.getTranslationsOf(srcString):
            results[result] = True

    # No translations found:
    if len(results) == 0:
        if config.config.debugTranslation:
            logger.log('Translation not found for "%s"' % srcString)
    return list(results.keys())


class TranslatableString(object):
    """
    Class representing a string that we want to match strings against, handling
    translation for us, by looking it up once at construction time.
    """

    def __init__(self, untranslatedString):
        """
        Constructor looks up the string in all of the translation databases, storing
        the various translations it finds.
        """
        try:  # python3 to get a plain always unicode string
            self.untranslatedString = str(untranslatedString)
        except UnicodeEncodeError:  # python2 to get non-unicode string for search comparions
            self.untranslatedString = untranslatedString.encode('utf-8')
        self.translatedStrings = translate(untranslatedString)

    def matchedBy(self, string):
        """
        Compare the test string against either the translation of the original
        string (or simply the original string, if no translation was found).
        """
        def stringsMatch(inS, outS):
            """
            Compares a regular expression to a string

            inS: the regular expression (or normal string)
            outS: the normal string to be compared against
            """
            inString = str(inS)
            outString = outS
            if inString == outString:
                return True
            inString = inString + '$'
            if inString[0] == '*':
                inString = "\\" + inString
            # Escape all parentheses, since grouping will never be needed here
            inString = re.sub('([\(\)])', r'\\\1', inString)
            match = re.match(inString, outString)
            matched = match is not None
            return matched

        matched = False
        # the 'ts' variable keeps track of whether we're working with
        # translated strings. it's only used for debugging purposes.
        #ts = 0
        # print string, str(self)
        for translatedString in self.translatedStrings:
            #ts = ts + 1
            matched = stringsMatch(translatedString, string)
            if not matched:
                matched = translatedString == string
            if matched:
                return matched
        # ts=0
        return stringsMatch(self.untranslatedString, string)

    def __str__(self):
        """
        Provide a meaningful debug version of the string (and the translation in
        use)
        """
        if len(self.translatedStrings) > 0:
            # build an output string, with commas in the correct places
            translations = ""
            for tString in self.translatedStrings:
                translations += '"%s", ' % tString
            result = '"%s" (%s)' % (
                self.untranslatedString, translations)
            return result
        else:
            return str('"%s"') % self.untranslatedString


def isMoFile(filename, language=''):
    """
    Does the given filename look like a gettext mo file?

    Optionally: Does the file also contain translations for a certain language,
    for example 'ja'?
    """
    if re.match('(.*)\\.mo$', filename):
        if not language:
            return True
        elif re.match('/usr/share/locale(.*)/%s(.*)/LC_MESSAGES/(.*)\\.mo$' % language, filename):
            return True
        else:
            return False
    else:
        return False


def loadAllTranslationsForLanguage(language):
    from dogtail import distro
    for moFile in distro.packageDb.getMoFiles(language):
        translationDbs.append(GettextTranslationDb(moFile))


def getMoFilesForPackage(packageName, language='', getDependencies=True):
    """
    Look up the named package and find all gettext mo files within it and its
    dependencies. It is possible to restrict the results to those of a certain
    language, for example 'ja'.
    """
    from dogtail import distro

    result = []
    for filename in distro.packageDb.getFiles(packageName):
        if isMoFile(filename, language):
            result.append(filename)

    if getDependencies:
        # Recurse:
        for dep in distro.packageDb.getDependencies(packageName):
            # We pass False to the inner call because getDependencies has already
            # walked the full tree
            result.extend(getMoFilesForPackage(dep, language, False))

    return result


def loadTranslationsFromPackageMoFiles(packageName, getDependencies=True):
    """
    Helper function which appends all of the gettext translation mo-files used by
    the package (and its dependencies) to the translation database list.
    """
    # Keep a list of mo-files that are already in use to avoid duplicates.
    moFiles = {}

    def load(packageName, language='', getDependencies=True):
        for moFile in getMoFilesForPackage(packageName, language, getDependencies):
            # Searching the popt mo-files for translations makes gettext bail out,
            # so we ignore them here. This is
            # https://bugzilla.redhat.com/bugzilla/show_bug.cgi?id=172155 .
            if not('popt.mo' in moFile or moFile in moFiles):
                try:
                    translationDbs.append(GettextTranslationDb(moFile))
                    moFiles[moFile] = None
                except (AttributeError, IndexError):
                    if config.config.debugTranslation:
                        #import traceback
                        # logger.log(traceback.format_exc())
                        logger.log("Warning: Failed to load mo-file for translation: " + moFile)

    # Hack alert:
    #
    # The following special-case is necessary for Ubuntu, since their
    # translations are shipped in a single huge package. The downside to
    # this special case, aside from the simple fact that there is one,
    # is that it makes automatic translations much slower.

    from dogtail import distro
    language = os.environ.get('LANGUAGE', os.environ['LANG'])[0:2]
    if isinstance(distro.distro, distro.Ubuntu):
        load('language-pack-gnome-%s' % language, language)
    load(packageName, language, getDependencies)



#!/usr/bin/env python3
# _*_ coding:utf-8 _*_

# SPDX-FileCopyrightText: 2023 UnionTech Software Technology Co., Ltd.

# SPDX-License-Identifier: GPL-2.0-only
# pylint: disable=C0114
# pylint: disable=C0301,W1514,R1721,R1710,W1514,C0103,R1710
import os
import csv

from src  import logger


class ReadCsv:
    """
    读取 csv 文件
    """

    __author__ = "Mikigo <huangmingqiang@uniontech.com>"

    @staticmethod
    def read_csv(csv_path):
        """
         通过标准库 csv 读取文件
        :param csv_path: 文件路径
        :return: 数据列表
        """
        if os.path.exists(csv_path):
            with open(csv_path, newline="") as csvfile:
                reader = csv.reader(csvfile)
                return [row for row in reader][1:]
        logger.error(f"{csv_path} is not exists!")

    @staticmethod
    def read_csv_by_str(csv_path, maxsplit=-1):
        """
         通过字符串处理读取字符串
        :param csv_path: 文件路径
        :param maxsplit: 最大分割次数
        :return: 数据列表
        """
        if os.path.exists(csv_path):
            with open(csv_path, "r") as f:
                txt_list = f.readlines()
            return [txt.strip().split(",", maxsplit=maxsplit) for txt in txt_list[1:]]
        logger.error(f"{csv_path} is not exists!")



#!/usr/bin/env python3
# _*_ coding:utf-8 _*_

# SPDX-FileCopyrightText: 2023 UnionTech Software Technology Co., Ltd.

# SPDX-License-Identifier: GPL-2.0-only
# pylint: disable=C0114,C0103
import re

from setting.globalconfig import GlobalConfig
from src  import logger
from src.cmdctl import CmdCtl
from src.custom_exception import ElementNotFound
from src.custom_exception import ApplicationStartError

try:
    from src.depends.dogtail.tree import SearchError
    from src.depends.dogtail.tree import root
    from src.depends.dogtail.tree import predicate
    from src.depends.dogtail.tree import config

    config.childrenLimit = 1000
    # config.logDebugToStdOut = False
    config.logDebugToFile = False
    config.searchCutoffCount = 2
    GlobalConfig.NO_DOGTAIL = False
except ModuleNotFoundError:
    GlobalConfig.NO_DOGTAIL = True

from src.mouse_key import MouseKey


class DogtailUtils(MouseKey):
    """
    通过属性进行元素定位和操作。
    """
    # pylint: disable=too-many-arguments,too-many-locals,too-many-public-methods
    __author__ = "Mikigo <huangmingqiang@uniontech.com>, Litao <litaoa@uniontech.com>"

    def __init__(
            self, name=None, description=None, number=-1, check_start=True, key: dict = None
    ):
        if GlobalConfig.NO_DOGTAIL:
            raise EnvironmentError("Dogtail 及其相关依赖存在问题,调用相关方法失败~")
        config.logDebugToStdOut = False
        self.name = name
        self.description = description
        try:
            if name:
                self.obj = root.application(self.name, self.description)
            else:
                self.obj = root
            if number > 0:
                self.obj = self.obj.findChildren(
                    predicate.GenericPredicate(**key)
                )[number]

        except SearchError:
            if check_start:
                search_app = CmdCtl.run_cmd(f"ps -ef | grep {self.name}")
                logger.error(search_app)
                raise ApplicationStartError(self.name) from SearchError

    def app_element(self, *args, **kwargs):
        """
         获取app元素的对象
        :return: 元素的对象
        """
        try:
            element = self.obj.child(*args, **kwargs, retry=False)
            logger.debug(f"{args, kwargs} 获取元素对象 <{element}>")
            return element
        except SearchError:
            raise ElementNotFound(*args, **kwargs) from SearchError

    def left_upper_corner_position(self, element) -> tuple:
        """
         获取元素左上角的坐标
        :param element: 元素名称
        :return: 元素左上角坐标
        """
        position = self.app_element(element).position
        logger.debug(f"获取元素 {element}元素左上角坐标 {position}")
        return position

    def element_size(self, *args, **kwargs) -> tuple:
        """
         获取元素的大小
        :return: 元素大小
        """
        size = self.app_element(*args, **kwargs).size
        logger.debug(f"元素{args, kwargs} 的大小 {size}")
        return size

    def right_upper_corner_position(self, element) -> tuple:
        """
         获取元素右上角的坐标
        :param element: 元素名称
        :return: 元素右上角坐标
        """
        _x = self.left_upper_corner_position(element)[0] + self.element_size(element)[0]
        _y = self.left_upper_corner_position(element)[1]
        logger.debug(f"获取元素 {element}, 右上角坐标 ({_x, _y})")
        return int(_x), int(_y)

    def element_center(self, element) -> tuple:
        """
         获取元素的中心位置
        :param element:
        :return: 元素中心坐标
        """
        _x, _y, _w, _h = self.app_element(element).extents
        _x = _x + _w / 2
        _y = _y + _h / 2
        logger.debug(f"获取元素中心坐标 ({_x, _y})")
        return _x, _y

    def element_click(self, element, button=1):
        """
         元素点击
        :param element: 应用的元素
        :param button: 1>left,2>middle,3>right
        :return: None
        """
        logger.debug(
            f"""{"左键" if button == 1 else f"{'右键' if button == 3 else '鼠标中健'}"} 点击元素 {element}"""
        )
        mouse_click = (
            self.click
            if button == 1
            else self.right_click
            if button == 3
            else self.middle_click
        )
        mouse_click(*self.element_center(element))

    def element_double_click(self, element):
        """
         元素双击
        :return: None
        """
        logger.debug(f"双击元素 {element}")
        self.double_click(*self.element_center(element))

    def element_point(self, element):
        """
         鼠标移动到元素上（位置是在元素的中心）
        :param element: 应用的元素
        :return: None
        """
        logger.debug(f"鼠标移至元素 {element} 中心")
        self.move_to(*self.element_center(element))

    @staticmethod
    def __evalx(expr, element, recursive):
        """evalx"""
        node = re.match(".*?[^\\\\]/", expr)
        if node:
            name = node.group().replace("\\/", "/")[:-1]
        else:
            return False
        if name == "*":
            element = element.children
        else:
            element = element.findChildren(
                predicate.GenericPredicate(name), recursive=recursive
            )
        return node, element

    def __trace(self, element, result, expr):
        """trace"""
        if expr.startswith("//"):
            name = expr[2:]
            node, element = self.__evalx(name, element, recursive=True)
        elif expr.startswith("/"):
            name = expr[1:]
            node, element = self.__evalx(name, element, recursive=False)
        else:
            return False
        try:
            next_node = name[node.end() - 1:]
            if next_node != "/":
                for i in element:
                    self.__trace(i, result, next_node)
            else:
                result += element
        except SearchError:
            raise ElementNotFound(expr) from SearchError
        return result

    def find_elements_by_attr(self, expr):
        """
         通过层级获取元素
        :param expr: 元素定位 $/xx.xxx//xxx,  $根节点  /当前子节点， //递归查找子节点
        :return: 元素对象
        """
        logger.debug(f"查找元素 expr={expr}")
        if expr == "$":
            return self.obj if isinstance(self.obj, list) else [self.obj]
        if not expr.startswith("$"):
            return False
        if not expr.endswith("/") or expr.endswith(r"\/"):
            expr = expr + "/"
        result = self.__trace(self.obj, [], expr[1:])
        logger.debug(f"元素 {result}")
        return result

    def find_element_by_attr(self, expr, index=0):
        """
         查找界面元素
        :param expr: 匹配格式
        :param index: 匹配结果索引
        :return: 元素对象
        """
        elements = self.find_elements_by_attr(expr)
        if not elements:
            raise ElementNotFound(expr)
        try:
            return elements[index]
        except IndexError:
            raise ElementNotFound(f"{expr}, index:{index}") from IndexError



#!/usr/bin/env python3
# _*_ coding:utf-8 _*_

# SPDX-FileCopyrightText: 2023 UnionTech Software Technology Co., Ltd.

# SPDX-License-Identifier: GPL-2.0-only
# pylint: disable=C0114
# pylint: disable=E0401,C0413,R0903,W0707,W0611
try:
    import cv2 as cv

    GET_OPENCV_FORM_RPC = False
except ModuleNotFoundError:
    GET_OPENCV_FORM_RPC = True

from pdocr_rpc import OCR as _OCR
from pdocr_rpc.conf import setting as ocr_setting

from setting import conf

ocr_setting.PORT = conf.OCR_PORT
ocr_setting.SERVER_IP = conf.OCR_SERVER_HOST
ocr_setting.NETWORK_RETRY = int(conf.OCR_NETWORK_RETRY)
ocr_setting.PAUSE = float(conf.OCR_PAUSE)
ocr_setting.TIMEOUT = float(conf.OCR_TIMEOUT)
ocr_setting.MAX_MATCH_NUMBER = int(conf.OCR_MAX_MATCH_NUMBER)


class OCRUtils(_OCR):
    """OCR识别的工具类"""




# pyflakes.ignore
from .pygtkcompat import (enable,
                          enable_gtk,
                          enable_vte,
                          enable_poppler,
                          enable_webkit,
                          enable_gudev,
                          enable_gst,
                          enable_goocanvas)


__all__ = ['enable',
           'enable_gtk',
           'enable_vte',
           'enable_poppler',
           'enable_webkit',
           'enable_gudev',
           'enable_gst',
           'enable_goocanvas']



#!/usr/bin/env python3
# _*_ coding:utf-8 _*_

# SPDX-FileCopyrightText: 2023 UnionTech Software Technology Co., Ltd.

# SPDX-License-Identifier: GPL-2.0-only
# pylint: disable=C0114

import dbus


class DbusUtils:
    """
    Obtain or operate the DBUS interface of the application through DBUS
    """

    def __init__(self, dbus_name=None, object_path=None, interface=None):
        """
        You can view the following parameters through d-feet.
        dbus_name
            eg.com.deepin.daemon.Appearance
        object_path
            eg./com/deepin/daemon/Appearance
        interface
            eg.com.deepin.daemon.Appearance

        Install d-feet using the command 'sudo apt install d-feet'
        """
        self.dbus_name = dbus_name
        self.object_path = object_path
        self.interface = interface
        self.session_dbus = dbus.SessionBus()
        self.system_dbus = dbus.SystemBus()

    def session_object_methods(self):
        """
         可以调用应用所具有的方法
        :return: 方法的对象
        """
        proxy_object = self.session_dbus.get_object(self.dbus_name, self.object_path)
        object_methods = dbus.Interface(proxy_object, self.interface)
        return object_methods

    def session_object_properties(self):
        """
         可以获取应用所具有的属性
        :return: 属性的对象
        """
        proxy_object = self.session_dbus.get_object(self.dbus_name, self.object_path)
        object_properties = dbus.Interface(proxy_object, dbus.PROPERTIES_IFACE)
        return object_properties

    def set_session_properties_value(self, property_name, set_value):
        """
         设置属性的值
        :param property_name: 属性名称
        :return: 属性的值
        """
        self.session_object_properties().Set(self.interface, property_name, set_value)

    def get_session_properties_value(self, property_name):
        """
         获取应用属性的值
        :param property_name: 属性名称
        :return: 属性的值
        """
        value = self.session_object_properties().Get(self.interface, property_name)
        return value

    def get_system_properties_value(self, property_name):
        """
         获取系统dbus属性的值
        :param property_name: 属性名称
        :return: 属性的值
        """
        proxy_object = self.system_dbus.get_object(self.dbus_name, self.object_path)
        object_propertie = dbus.Interface(proxy_object, dbus.PROPERTIES_IFACE)
        return object_propertie.Get(self.interface, property_name)

    def system_object_methods(self):
        """
         可以调用应用所具有的方法
        :return: 方法的对象
        """
        proxy_object = self.system_dbus.get_object(self.dbus_name, self.object_path)
        object_methods = dbus.Interface(proxy_object, self.interface)
        return object_methods



#!/usr/bin/env python3
# _*_ coding:utf-8 _*_

# SPDX-FileCopyrightText: 2023 UnionTech Software Technology Co., Ltd.

# SPDX-License-Identifier: GPL-2.0-only
# pylint: disable=C0114
# pylint: disable=C0301,R0913,R0914,W0613,R0912,R0915,E0401,C0413,C0103,C0116
import json
import re
import sys
from collections import Counter
from os import chdir
from os import environ
from os import listdir
from os import makedirs
from os import system
from os.path import exists
from os.path import expanduser
from os.path import isfile
from os.path import join
from time import sleep
from tkinter import Tk

import pytest
from allure_custom import AllureCustom
from allure_custom.conf import setting as al_setting

from setting.globalconfig import GetCfg
from setting.globalconfig import GlobalConfig

al_setting.html_title = GlobalConfig.REPORT_TITLE
al_setting.report_name = GlobalConfig.REPORT_NAME
al_setting.report_language = GlobalConfig.REPORT_LANGUAGE

import letmego

from src import logger
from src.rtk._base import Args
from src.requestx import RequestX
from src.rtk._base import transform_app_name

environ["DISPLAY"] = ":0"


class LocalRunner:
    """
    本地执行器
    """
    __author__ = "Mikigo <huangmingqiang@uniontech.com>"

    def __init__(
            self,
            app_name=None,
            keywords=None,
            tags=None,
            report_formats=None,
            max_fail=None,
            reruns=None,
            record_failed_case=None,
            clean=None,
            log_level=None,
            timeout=None,
            debug=False,
            noskip=None,
            ifixed=None,
            send_pms=None,
            task_id=None,
            suite_id=None,
            trigger=None,
            resolution=None,
            case_file=None,
            deb_path=None,
            pms_user=None,
            pms_password=None,
            pms_info_file=None,
            top=None,
            lastfailed=None,
            duringfail=None,
            repeat=None,
            project_name=None,
            build_location=None,
            line=None,
            collection_only=None,
            autostart=None,
            export_csv_file=None,
            **kwargs,
    ):
        logger("INFO")
        try:
            github_tags = RequestX().open_url(
                f"https://api.github.com/repos/linuxdeepin/deepin-autotest-framework/tags",
                timeout=1
            )
            latest_tag = json.loads(github_tags)[0].get("name")
            if GlobalConfig.current_tag != latest_tag:
                print(f"YouQu最新版本为: {latest_tag}，当前使用版本为: {GlobalConfig.current_tag}")
                print(f"建议使用：sudo pip3 install youqu=={latest_tag} 升级版本")
        except Exception:
            pass
        self.default = {
            Args.app_name.value: transform_app_name(app_name if app_name or case_file else GlobalConfig.APP_NAME),
            Args.keywords.value: keywords or GlobalConfig.KEYWORDS,
            Args.tags.value: tags or GlobalConfig.TAGS,
            Args.report_formats.value: report_formats or GlobalConfig.REPORT_FORMAT,
            Args.max_fail.value: max_fail or GlobalConfig.MAX_FAIL,
            Args.reruns.value: reruns or GlobalConfig.RERUN,
            Args.record_failed_case.value: record_failed_case or GlobalConfig.RECORD_FAILED_CASE,
            Args.clean.value: clean or GlobalConfig.CLEAN_ALL,
            Args.log_level.value: log_level or GlobalConfig.LOG_LEVEL,
            Args.timeout.value: timeout or GlobalConfig.CASE_TIME_OUT,
            Args.debug.value: debug or GlobalConfig.DEBUG,
            Args.noskip.value: noskip or GlobalConfig.NOSKIP,
            Args.ifixed.value: ifixed or GlobalConfig.IFIXED,
            Args.send_pms.value: send_pms or GlobalConfig.SEND_PMS,
            Args.task_id.value: task_id or GlobalConfig.TASK_ID,
            Args.suite_id.value: suite_id or GlobalConfig.SUITE_ID,
            Args.trigger.value: trigger or GlobalConfig.TRIGGER,
            Args.resolution.value: resolution or GlobalConfig.RESOLUTION,
            Args.case_file.value: case_file or GlobalConfig.CASE_FILE,
            Args.deb_path.value: deb_path or GlobalConfig.DEB_PATH,
            Args.duringfail.value: duringfail or GlobalConfig.DURING_FAIL,
            Args.repeat.value: repeat or GlobalConfig.REPEAT,
            Args.top.value: top or GlobalConfig.TOP,
            Args.pms_user.value: (pms_user or GlobalConfig.PMS_USER)
            if not pms_info_file
            else None,
            Args.pms_password.value: (pms_password or GlobalConfig.PMS_PASSWORD)
            if not pms_info_file
            else None,
            Args.pms_info_file.value: pms_info_file,
            Args.autostart.value: autostart or GlobalConfig.AUTOSTART,
        }
        self.lastfailed = lastfailed
        self.project_name = project_name
        self.build_location = build_location
        self.line = line
        self.collection_only = collection_only
        self.export_csv_file = export_csv_file or GlobalConfig.EXPORT_CSV_FILE

        if not (self.default.get(Args.debug.value) or self.collection_only):
            screen = Tk()
            x = screen.winfo_screenwidth()
            y = screen.winfo_screenheight()
            if self.default.get(Args.resolution.value) not in (f"{x}x{y}", "no"):
                raise ValueError(f"当前分辨率为：{x}x{y},您配置的分辨率为：{GlobalConfig.RESOLUTION}")

    @property
    def export_default(self):
        return self.default

    @staticmethod
    def make_dir(dirs):
        """make_dir"""
        try:
            dirs = expanduser(dirs)
            if not exists(dirs):
                makedirs(dirs)
        except FileExistsError:
            pass

    # 报告
    @classmethod
    def make_allure_report(cls, cmd, fmt, proj_path=None):
        """make_allure_report"""
        allure_report_path = f"{proj_path}/report/allure"
        if proj_path is None:
            allure_report_path = join(GlobalConfig.ALLURE_REPORT_PATH, fmt)
            cls.make_dir(allure_report_path)
        cmd.append(f"--alluredir={allure_report_path}")

    @classmethod
    def make_xml_report(cls, app_dir, case_file, cmd, fmt, proj_path=None):
        """make_xml_report"""
        xml_report_path = f"{proj_path}/report/xml"
        if proj_path is None:
            xml_report_path = join(GlobalConfig.XML_REPORT_PATH, fmt)
            cls.make_dir(xml_report_path)
        report_name = (
            case_file.replace("/", "_").replace(".", "_")
            if case_file
            else app_dir
        )
        cmd.append(
            f"--junit-xml={xml_report_path}/{report_name}{'-' if report_name else ''}{GlobalConfig.TIME_STRING}.xml"
        )

    def create_pytest_cmd(self, app_dir, default=None, proj_path=None):
        """
         创建 Pytest 及其命令行参数
        :param app_dir:
        :return:
        """
        if default is None:
            default = self.default
        keywords_or_marker = True
        cmd = ["pytest"]
        if self.lastfailed:
            cmd.append("--lf")
            keywords_or_marker = False
        # 通过文件存放pms信息,Jenkins环境下不希望明文显示密码等信息,可信息存放在文件中
        elif default.get(Args.pms_info_file.value):
            pms_info_file_path = (
                f"{GlobalConfig.ROOT_DIR}/{default.get(Args.pms_info_file.value)}"
            )
            if not exists(pms_info_file_path):
                logger.error(pms_info_file_path)
                raise FileNotFoundError
            pms_cfg = GetCfg(pms_info_file_path, "pms")
            cmd.extend(["--pms_user", pms_cfg.get("PMS_USER", default="")])
            cmd.extend(["--pms_password", pms_cfg.get("PMS_PASSWORD", default="")])
            keywords_or_marker = False
        # 通过pms测试套执行用例
        elif default.get(Args.pms_user.value) and default.get(
                Args.pms_password.value
        ):
            cmd.extend(["--pms_user", default.get(Args.pms_user.value)])
            cmd.extend(["--pms_password", default.get(Args.pms_password.value)])
            keywords_or_marker = False
        # 通过本地测试套
        elif default.get(Args.case_file.value):
            file_path = (
                f"{GlobalConfig.ROOT_DIR}/{default.get(Args.case_file.value)}"
            )
            if (not exists(file_path)) or (not isfile(file_path)):
                logger.error(f"{file_path} 文件不存在.")
                raise FileNotFoundError
            if not default.get(Args.case_file.value).endswith(".txt"):
                logger.error(f"测试套文件{default.get(Args.case_file.value)},不是一个txt文件")
                raise FileNotFoundError
            logger.info(f"本地测试套文件：{file_path}")
            with open(file_path, "r", encoding="utf-8") as _f:
                txt_list = _f.readlines()
            if not txt_list:
                logger.error(f"{file_path},文件为空")
                raise ValueError
            taglines = [txt.strip() for txt in txt_list if txt]
            cmd.extend(taglines)

        if keywords_or_marker:
            if default.get(Args.keywords.value):
                self.set_recursion_limit(default.get(Args.keywords.value))
                cmd.extend(["-k", f"'{default.get(Args.keywords.value)}'"])
            if default.get(Args.tags.value):
                self.set_recursion_limit(default.get(Args.tags.value))
                cmd.extend(["-m", f"'{default.get(Args.tags.value)}'"])
        if app_dir and app_dir != GlobalConfig.APPS_PATH:
            cmd.extend(["--app_name", app_dir])
        if self.export_csv_file:
            cmd.extend(["--export_csv_file", self.export_csv_file, "--verbosity=-1"])
        if self.collection_only:
            cmd.append("--co")
            return cmd

        if default.get(Args.noskip.value):
            cmd.extend(["--noskip", "yes"])
        if default.get(Args.ifixed.value):
            cmd.extend(["--ifixed", "yes"])
        if default.get(Args.send_pms.value):
            cmd.extend(["--send_pms", default.get(Args.send_pms.value)])
        if default.get(Args.suite_id.value):
            cmd.extend(["--suite_id", default.get(Args.suite_id.value)])
        if default.get(Args.task_id.value):
            cmd.extend(["--task_id", default.get(Args.task_id.value)])
        if default.get(Args.send_pms.value) and default.get(Args.trigger.value):
            cmd.extend(["--trigger", default.get(Args.trigger.value)])

        cmd.extend(
            [
                f"--max_fail={float(default.get(Args.max_fail.value))}",
                f"--reruns={default.get(Args.reruns.value)}",
                f"--record_failed_case={default.get(Args.record_failed_case.value)}",
                f"--clean={default.get(Args.clean.value)}",
                f"--log_level={default.get(Args.log_level.value)}",
                f"--timeout={default.get(Args.timeout.value)}",
            ]
        )
        if default.get(Args.duringfail.value):
            cmd.append("--duringfail")
        if default.get(Args.top.value):
            cmd.extend(["--top", default.get(Args.top.value)])
        if default.get(Args.autostart.value):
            GlobalConfig.LETMEGO_DEBUG = False
            cmd.extend(["--autostart", default.get(Args.autostart.value)])
        if default.get(Args.repeat.value):
            cmd.extend(["--repeat", default.get(Args.repeat.value)])
        if self.line:
            cmd.extend(["--line", self.line])

        report_formats = default.get(Args.report_formats.value)
        if report_formats:
            report_formats = [i.strip() for i in report_formats.split(",")]
            # allure
            if (GlobalConfig.ReportFormat.ALLURE in report_formats) and (
                    GlobalConfig.ReportFormat.JSON not in report_formats
            ):
                self.make_allure_report(cmd, GlobalConfig.ReportFormat.ALLURE, proj_path)
            # xml
            if GlobalConfig.ReportFormat.XML in report_formats:
                self.make_xml_report(
                    app_dir,
                    default.get(Args.case_file.value),
                    cmd,
                    GlobalConfig.ReportFormat.XML,
                    proj_path,
                )
            # json
            if (GlobalConfig.ReportFormat.ALLURE not in report_formats) and (
                    GlobalConfig.ReportFormat.JSON in report_formats
            ):
                self.make_allure_report(cmd, GlobalConfig.ReportFormat.ALLURE, proj_path)
                self.make_dir(
                    join(GlobalConfig.REPORT_PATH, GlobalConfig.ReportFormat.JSON)
                )
            # allure json
            if (GlobalConfig.ReportFormat.ALLURE in report_formats) and (
                    GlobalConfig.ReportFormat.JSON in report_formats
            ):
                self.make_allure_report(cmd, GlobalConfig.ReportFormat.ALLURE, proj_path)
                self.make_dir(
                    join(GlobalConfig.REPORT_PATH, GlobalConfig.ReportFormat.JSON)
                )
        return cmd

    @staticmethod
    def set_recursion_limit(strings):
        len_tags = len(re.split("or |and |not ", strings))
        if len_tags >= 999:
            sys.setrecursionlimit(len_tags + 100)

    def change_working_dir(self):
        """
         切换工作区间
        :param app_name:
        :return:
        """
        app_name: str = self.default.get(Args.app_name.value)
        if app_name:
            applications = listdir(GlobalConfig.APPS_PATH)
            for working_dir in applications:
                if working_dir == app_name:
                    case_path = f"{GlobalConfig.APPS_PATH}/{working_dir}"
                    print(f"WorkSpace: \n{case_path}")
                    chdir(case_path)
                    return working_dir
            raise NotADirectoryError(f"apps 目录下未找到指定的 {app_name}")
        return GlobalConfig.APPS_PATH

    def local_run(self):
        """
         执行用例
        :return:
        """
        if not self.default.get(Args.autostart.value):
            # 备份 allure 报告
            allure_report_path = join(GlobalConfig.ALLURE_REPORT_PATH, "allure")
            allure_report_back_path = join(
                GlobalConfig.ALLURE_REPORT_PATH, "allure_back", GlobalConfig.TIME_STRING
            )
            if exists(allure_report_path) and listdir(allure_report_path):
                makedirs(allure_report_back_path)
                system(f"mv {allure_report_path}/* {allure_report_back_path}/")

        app_dir = self.change_working_dir()
        run_test_cmd_list = self.create_pytest_cmd(app_dir)

        if not self.default.get(Args.pms_info_file.value):
            ...
            # print(f'Running: \n{" ".join(run_test_cmd_list)}')
        if self.default.get(Args.debug.value):
            logger.info("Debug 模式不执行用例!")
            return
        pytest.main([i.strip("'") for i in run_test_cmd_list[1:]])

        if self.collection_only:
            return
        if self.project_name and self.build_location and self.line:
            self.write_json(
                project_name=self.project_name,
                build_location=self.build_location,
                line=self.line,
            )
        allure_report_path = join(
            GlobalConfig.ALLURE_REPORT_PATH, GlobalConfig.ReportFormat.ALLURE
        )
        allure_html_report_path = join(GlobalConfig.ALLURE_REPORT_PATH, "allure_html")

        json_report_path = join(
            GlobalConfig.ALLURE_REPORT_PATH, GlobalConfig.ReportFormat.JSON
        )

        if self.default.get(Args.report_formats.value):
            report_formats = [
                i.strip()
                for i in self.default.get(Args.report_formats.value).split(",")
            ]
            if (GlobalConfig.ReportFormat.ALLURE in report_formats) or (
                    GlobalConfig.ReportFormat.JSON in report_formats
            ):
                if exists(allure_html_report_path):
                    system(f"rm -rf {allure_html_report_path}")
                makedirs(allure_html_report_path)

                AllureCustom.gen(allure_report_path, allure_html_report_path)

                if GlobalConfig.ReportFormat.JSON in report_formats:
                    system(
                        f"cp {allure_html_report_path}/widgets/suites.json "
                        f"{json_report_path}/"
                        f"result_{self.default.get(Args.app_name.value)}_{GlobalConfig.TIME_STRING}_{GlobalConfig.HOST_IP.replace('.', '')}.json"
                    )

        if exists(letmego.conf.setting.RUNNING_MAN_FILE):
            letmego.unregister_autostart_service()
            letmego.clean_running_man(
                copy_to=f"{GlobalConfig.REPORT_PATH}/_running_man_{GlobalConfig.TIME_STRING}.log"
            )

    @classmethod
    def get_result(cls, ci_result):
        """
         获取结果
        :return:
        """
        with open(ci_result, "r", encoding="utf-8") as _f:
            results_dict = json.load(_f)
        res = Counter([results_dict.get(i).get("result") for i in results_dict])
        total = sum(res.values())
        skiped = res.get("skip", 0)
        total = total - skiped
        passed = res.get("pass", 0)
        failed = total - passed
        pass_rate = f"{round((passed / total) * 100, 1)}%" if passed else "0%"
        return total, failed, passed, pass_rate

    @classmethod
    def write_json(cls, project_name=None, build_location=None, line=None):
        """
         写json报告
        {
          "project_name": "",
          "build_location": "",
          "line": "",
          "total": "",
          "fail": "",
          "block": "",
          "pass": "",
          "pass_rate": ""
        }
        """
        json_tpl_path = f"{GlobalConfig.SETTING_PATH}/template/ci.json"
        if not exists(json_tpl_path):
            raise FileNotFoundError
        with open(json_tpl_path, "r", encoding="utf-8") as _f:
            results = json.load(_f)

        results["project_name"] = project_name
        results["build_location"] = build_location
        results["line"] = line
        ci_result_path = f"{GlobalConfig.ROOT_DIR}/ci_result.json"
        if not exists(ci_result_path):
            return

        (
            results["total"],
            results["fail"],
            results["pass"],
            results["pass_rate"],
        ) = cls.get_result(ci_result_path)

        json_res_path = f"{GlobalConfig.ROOT_DIR}/{project_name}_at.json"
        with open(json_res_path, "w+", encoding="utf-8") as _f:
            _f.write(json.dumps(results, indent=2, ensure_ascii=False))
        sleep(1)
        with open(json_res_path, "r", encoding="utf-8") as _f:
            print("CICD数据结果:\n", _f.read())

    def install_deb(self):
        """
         安装本地 deb 包
        :return:
        """
        logger.info("安装deb包")
        system(
            f"cd {self.default.get(Args.deb_path.value)}/ && echo {GlobalConfig.PASSWORD} | sudo -S dpkg -i *.deb"
        )
        logger.info("deb包安装完成")


if __name__ == "__main__":
    LocalRunner().local_run()



#!/usr/bin/env python3
# _*_ coding:utf-8 _*_

# SPDX-FileCopyrightText: 2023 UnionTech Software Technology Co., Ltd.

# SPDX-License-Identifier: GPL-2.0-only
# pylint: disable=C0114
import os

for root, dirs, files in os.walk(os.path.abspath("../../apps")):
    for file in files:
        if file == "requirement.txt":
            print(f"{root}/{file}")



#!/usr/bin/env python3
# _*_ coding:utf-8 _*_

# SPDX-FileCopyrightText: 2023 UnionTech Software Technology Co., Ltd.

# SPDX-License-Identifier: GPL-2.0-only
import os
from time import sleep

from setting.globalconfig import GlobalConfig

# wayland PATH
if GlobalConfig.IS_WAYLAND:
    os.environ["QT_WAYLAND_SHELL_INTEGRATION"] = "kwayland-shell"
    os.environ["XDG_SESSION_DESKTOP"] = "Wayland"
    os.environ["XDG_SESSION_TYPE"] = "wayland"
    os.environ["WAYLAND_DISPLAY"] = "wayland-0"
    os.environ["GDMSESSION"] = "Wayland"
    os.environ["DBUS_SESSION_BUS_ADDRESS"] = "unix:path=/run/user/1000/bus"

from platform import machine
import ctypes


class Geometry(ctypes.Structure):
    """sub structure"""
    _fields_ = [
        ("x", ctypes.c_int),
        ("y", ctypes.c_int),
        ("width", ctypes.c_int),
        ("height", ctypes.c_int),
    ]


class WindowState(ctypes.Structure):
    """window structure"""
    _fields_ = [
        ("pid", ctypes.c_int),  # 4
        ("windowId", ctypes.c_int),  # 4
        ("resourceName", ctypes.c_char * 256),  # 256
        ("Geometry", Geometry),  # 16
        ("isMinimized", ctypes.c_bool),  # 4
        ("isFullScreen", ctypes.c_bool),
        ("isActive", ctypes.c_bool),
        ("splitable", ctypes.c_int),  # 4
        ("uuid", ctypes.c_char * 256),  # 256
    ]


class dtk_array(ctypes.Structure):
    _fields_ = [
        ("size", ctypes.c_size_t),
        ("alloc", ctypes.c_size_t),
        ("data", ctypes.POINTER(WindowState)),
    ]


class WindowStructure(ctypes.Structure):
    """window structure"""
    _fields_ = [
        ("pid", ctypes.c_int),
        ("windowId", ctypes.c_int),
        ("resourceName", ctypes.c_char * 256),
        ("Geometry", Geometry),
        ("isMinimized", ctypes.c_bool),
        ("isFullScreen", ctypes.c_bool),
        ("isActive", ctypes.c_bool),
    ]


class WaylandWindowInfo:
    """获取窗口信息"""

    def __init__(self):
        self.library = ctypes.cdll.LoadLibrary(f"/usr/lib/{machine()}-linux-gnu/libdtkwmjack.so")
        if not GlobalConfig.DTK_DISPLAY:
            self.init_dtk_display()
            GlobalConfig.DTK_DISPLAY = True

    def init_dtk_display(self):
        self.library.InitDtkWmDisplay()

    def destory_dtk_display(self):
        self.library.DestoryDtkWmDisplay()

    def _window_info(self):
        """窗口信息"""
        self.library.GetWindowFromPoint.restype = ctypes.c_int
        wid = self.library.GetWindowFromPoint()
        self.library.GetWindowState.restype = ctypes.POINTER(WindowStructure)
        ws = self.library.GetWindowState(wid)
        window_info = ws.contents.Geometry
        resourceName = ws.contents.resourceName.decode("utf-8")
        if not resourceName:
            resourceName = os.popen(f"cat /proc/{ws.contents.pid}/cmdline").read().strip("\x00")
        return {
            "name": resourceName,
            "wininfo": (
                window_info.x,
                window_info.y,
                window_info.width,
                window_info.height
            ),
        }

    def window_info(self):
        """窗口信息"""
        self.library.GetAllWindowStatesList.restype = ctypes.POINTER(dtk_array)
        _e = None
        for _ in range(3):
            try:
                get_all_window_states_list = self.library.GetAllWindowStatesList()
                range_index = get_all_window_states_list.contents.size / 544
                break
            except ValueError as e:
                _e = e
                sleep(1)
        else:
            raise ValueError(_e)
        res = {}
        for i in range(int(range_index)):
            window_info = get_all_window_states_list.contents.data[i]
            resource_name = window_info.resourceName.decode("utf-8")
            if not resource_name:
                resource_name = os.popen(f"cat /proc/{window_info.pid}/cmdline").read().strip("\x00")
            _info = {
                "location": (
                    window_info.Geometry.x,
                    window_info.Geometry.y,
                    window_info.Geometry.width,
                    window_info.Geometry.height
                ),
                "pid": window_info.pid,
                "window_id": window_info.windowId,
                "uuid": window_info.uuid.decode("utf-8"),
                "is_minimized": window_info.isMinimized,
                "is_full_screen": window_info.isFullScreen,
                "is_active": window_info.isActive,
            }
            if res.get(resource_name) is None:
                res[resource_name] = _info
            else:
                if isinstance(res.get(resource_name), dict):
                    res[resource_name] = [res.get(resource_name), _info]
                elif isinstance(res.get(resource_name), list):
                    res[resource_name].append(_info)
        return res


if __name__ == '__main__':
    wwininfo = WaylandWindowInfo()
    wwininfo.library.InitDtkWmDisplay()
    for i in range(100):
        print(wwininfo.window_info())
        sleep(1)
    wwininfo.library.DestoryDtkWmDisplay()



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function, unicode_literals

"""
Handles versioning of software packages
"""
__author__ = 'Dave Malcolm <dmalcolm@redhat.com>'


class Version(object):
    """
    Class representing a version of a software package.
    Stored internally as a list of subversions, from major to minor.
    Overloaded comparison operators ought to work sanely.
    """

    def __init__(self, versionList):
        self.versionList = versionList

    def fromString(versionString):
        """
        Parse a string of the form number.number.number
        """
        return Version(list(map(int, versionString.split("."))))
    fromString = staticmethod(fromString)

    def __str__(self):
        return ".".join(map(str, self.versionList))

    def __getNum(self):
        tmpList = list(self.versionList)

        while len(tmpList) < 5:
            tmpList += [0]

        num = 0
        for i in range(len(tmpList)):
            num *= 1000
            num += tmpList[i]
        return num

    def __lt__(self, other):
        return self.__getNum() < other.__getNum()

    def __le__(self, other):
        return self.__getNum() <= other.__getNum()

    def __eq__(self, other):
        return self.__getNum() == other.__getNum()

    def __ne__(self, other):
        return self.__getNum() != other.__getNum()

    def __gt__(self, other):
        return self.__getNum() > other.__getNum()

    def __ge__(self, other):
        return self.__getNum() >= other.__getNum()



"""
    Python cFonts
    =============
    Sexy fonts for the console.

    :license: GNU GPLv2
    :author: Frost Ming<mianghong@gmail.com>
"""
__all__ = ["say", "render"]

from .core import render, say



#!/usr/bin/env python3
# _*_ coding:utf-8 _*_

# SPDX-FileCopyrightText: 2023 UnionTech Software Technology Co., Ltd.

# SPDX-License-Identifier: GPL-2.0-only
# pylint: disable=C0114
# pylint: disable=C0301,C0115,R0903,C0103,C0201,R1710,R0914,W1514,R0914,R1702
import json
import os
from configparser import ConfigParser

from setting.globalconfig import FixedCsvTitle
from setting.globalconfig import GetCfg
from setting.globalconfig import GlobalConfig
from src import logger
from src.pms._base import MAX_CASE_NUMBER
from src.pms._base import _Base
from src.pms._base import _unicode_to_cn
from src.rtk._base import transform_app_name


class Pms2Csv(_Base):
    """获取pms数据同步到本地csv文件"""

    __author__ = "mikigo<huangmingqiang@uniontech.com>"

    config_error_log = "请检查您传递的 '命令行参数' 或 setting/globalconfig.ini 里的配置项"

    def __init__(self, app_name=None, user=None, password=None, pms_link_csv=None):
        super().__init__(user=user, password=password)
        self.walk_dir = (
            f"{GlobalConfig.APPS_PATH}/{transform_app_name(app_name)}"
            if app_name
            else GlobalConfig.APPS_PATH
        )

        conf = ConfigParser()
        conf.read(GlobalConfig.GLOBAL_CONFIG_FILE_PATH)
        ini_csv_names = conf.options("pmsctl-pms_link_csv")
        ini_csv_pms_map = GetCfg(
            GlobalConfig.GLOBAL_CONFIG_FILE_PATH, "pmsctl-pms_link_csv"
        )
        cli_csv_pms_map = {}
        cli_csv_names = []
        if pms_link_csv:
            _cli_csv_names = pms_link_csv.split("/")
            for i in _cli_csv_names:
                pls = i.split(":")
                if len(pls) != 2:
                    raise ValueError("--pms_link_csv 参数的值可能有问题")
                csv_name, pms_product_id = pls
                cli_csv_names.append(csv_name.strip())
                cli_csv_pms_map[csv_name.strip()] = pms_product_id.strip()

        self.csv_names = cli_csv_names or ini_csv_names
        self.csv_link_cfg = cli_csv_pms_map or ini_csv_pms_map

        if not self.csv_names:
            raise ValueError(self.config_error_log)

        self.pms_mark = GetCfg(
            f"{GlobalConfig.SETTING_PATH}/pmsmark.ini", "pms-mark-to-csv-mark"
        )

    def get_data_from_pms(self, app_case_id):
        """获取pms上数据"""
        if not app_case_id:
            raise ValueError(self.config_error_log)
        case_url = (
            f"https://pms.uniontech.com/{GlobalConfig.CASE_FROM}-browse-"
            f'{app_case_id}-{"-" if GlobalConfig.CASE_FROM == "testcase" else ""}all-0-id_desc-0-{MAX_CASE_NUMBER}.json'
        )
        res = self.rx.open_url(case_url, timeout=10)
        res_str = _unicode_to_cn(res)

        try:
            res_dict = json.loads(res_str)
        except json.decoder.JSONDecodeError:
            logger.error(f"获取pms数据失败, {self.config_error_log}")
            return

        cases = res_dict.get("data").get("cases")
        res_data = {}
        for i in cases:
            case = cases.get(i)
            case_id = case.get("id")
            case_level = case.get("pri")
            case_type = self.pms_mark.get(case.get("type"))
            case_from = case.get("caseSource")
            device_type = case.get("deviceType")
            online_obj = case.get("lineCD")
            if case_type:
                res_data[case_id] = {
                    "case_level": f"L{case_level}",
                    "case_type": case_type,
                    "case_from": "BUG" if case_from == "是" else "",
                    "device_type": device_type.split("(")[0]
                    if device_type and device_type != "null"
                    else "",
                    "online_obj": "CICD" if online_obj == "是" else "",
                    "skip_reason": "skip-下线CD" if online_obj == "否" else None
                }
        if not res_data:
            logger.error(f"未从pms获取到数据, {self.config_error_log}")
            raise ValueError
        return res_data

    def read_csv(self):
        """读取本地csv文件数据"""
        csv_path_dict = {}
        csv_bak_path = f"{GlobalConfig.REPORT_PATH}/pms2csv_back"
        if not os.path.exists(csv_bak_path):
            os.makedirs(csv_bak_path)
        for root, _, files in os.walk(self.walk_dir):
            for file in files:
                if (
                    file.endswith(".csv")
                    and os.path.splitext(file)[0] in self.csv_names
                ):
                    csv_path_dict[os.path.splitext(file)[0]] = f"{root}/{file}"
                    os.system(
                        f"cp {root}/{file} {csv_bak_path}/{GlobalConfig.TIME_STRING}_{file}"
                    )
        if not csv_path_dict:
            raise ValueError(f"{self.walk_dir} 目录下未找对应的到csv文件，{self.config_error_log}")

        res_tags = {}
        csv_heads_dict = {}
        for csv_name in csv_path_dict:
            with open(csv_path_dict.get(csv_name), "r", encoding="utf-8") as f:
                txt_list = f.readlines()
            csv_heads = txt_list[0].strip().split(",")

            csv_head_index_map = {}
            for index, title in enumerate(csv_heads):
                for i in FixedCsvTitle:
                    if i.value == title.strip():
                        csv_head_index_map[i.name] = {
                            "head_name": i.value,
                            "head_index": index,
                        }

            taglines = [txt.strip().split(",") for txt in txt_list[1:]]
            id_tags_dict = {i[0]: i for i in taglines if i[0]}
            res_tags[csv_name] = id_tags_dict
            csv_heads_dict[csv_name] = csv_head_index_map
        return res_tags, csv_heads_dict, csv_path_dict

    def compare_pms_to_csv(self):
        """对比pms上数据和本地csv文件数据"""
        (res_tags, csv_heads_dict, csv_path_dict) = self.read_csv()
        new_csv_file_tags = {}
        for csv_name in res_tags:
            product_id = self.csv_link_cfg.get(csv_name)
            pms_tags_dict = self.get_data_from_pms(product_id)
            if pms_tags_dict is None:
                continue
            logger.info(f"csv_name: {csv_name}")
            logger.info(f"product_id: {product_id}")
            csv_tags_dict = res_tags.get(csv_name)
            csv_head_dict = csv_heads_dict.get(csv_name)

            pms_case_id_index = case_level_index = case_type_index = None
            case_from_index = device_type_index = online_obj_index = None
            skip_reason_index = None

            pms_case_id_name = csv_head_dict.get(FixedCsvTitle.pms_case_id.name)
            if pms_case_id_name:
                pms_case_id_index = pms_case_id_name.get("head_index")

            case_level_name = csv_head_dict.get(FixedCsvTitle.case_level.name)
            if case_level_name:
                case_level_index = case_level_name.get("head_index")

            case_type_name = csv_head_dict.get(FixedCsvTitle.case_type.name)
            if case_type_name:
                case_type_index = case_type_name.get("head_index")

            case_from_name = csv_head_dict.get(FixedCsvTitle.case_from.name)
            if case_from_name:
                case_from_index = case_from_name.get("head_index")

            device_type_name = csv_head_dict.get(FixedCsvTitle.device_type.name)
            if device_type_name:
                device_type_index = device_type_name.get("head_index")

            online_obj_name = csv_head_dict.get(FixedCsvTitle.online_obj.name)
            if online_obj_name:
                online_obj_index = online_obj_name.get("head_index")

            skip_reason_name = csv_head_dict.get(FixedCsvTitle.skip_reason.name)
            if skip_reason_name:
                skip_reason_index = skip_reason_name.get("head_index")

            new_csv_tags = []
            new_csv_tags.append(
                [i.get("head_name") for i in list(csv_head_dict.values())]
            )
            for csv_case_id in csv_tags_dict:
                for pms_case_id in pms_tags_dict:
                    if pms_case_id == csv_case_id:
                        pms_tags = pms_tags_dict.get(pms_case_id)
                        case_level = pms_tags.get("case_level")
                        case_type = pms_tags.get("case_type")
                        case_from = pms_tags.get("case_from")
                        device_type = pms_tags.get("device_type")
                        online_obj = pms_tags.get("online_obj")
                        skip_reason = pms_tags.get("skip_reason")
                        flag = False
                        if (
                            pms_case_id_index
                            and csv_tags_dict[csv_case_id][pms_case_id_index]
                            != pms_case_id
                        ):
                            csv_tags_dict[csv_case_id][pms_case_id_index] = pms_case_id
                            flag = True
                        if (
                            case_level_index
                            and csv_tags_dict[csv_case_id][case_level_index]
                            != case_level
                        ):
                            csv_tags_dict[csv_case_id][case_level_index] = case_level
                            flag = True
                        if (
                            case_type_index
                            and csv_tags_dict[csv_case_id][case_type_index] != case_type
                        ):
                            csv_tags_dict[csv_case_id][case_type_index] = case_type
                            flag = True
                        if (
                            case_from_index
                            and csv_tags_dict[csv_case_id][case_from_index] != case_from
                        ):
                            csv_tags_dict[csv_case_id][case_from_index] = case_from
                            flag = True
                        if (
                            device_type_index
                            and csv_tags_dict[csv_case_id][device_type_index]
                            != device_type
                        ):
                            csv_tags_dict[csv_case_id][device_type_index] = device_type
                            flag = True
                        if (
                            online_obj_index
                            and csv_tags_dict[csv_case_id][online_obj_index]
                            != online_obj
                        ):
                            csv_tags_dict[csv_case_id][online_obj_index] = online_obj
                            flag = True
                        if (
                            skip_reason_index
                            and csv_tags_dict[csv_case_id][skip_reason_index]
                            != skip_reason
                        ):
                            if skip_reason:
                                csv_tags_dict[csv_case_id][skip_reason_index] = skip_reason
                                flag = True

                        new_tags = csv_tags_dict[csv_case_id]
                        if flag:
                            logger.info(
                                f"pms case id: {pms_case_id}, new tags:{new_tags}"
                            )
                        new_csv_tags.append(new_tags)
                        break
                else:
                    new_csv_tags.append(csv_tags_dict[csv_case_id])

            new_csv_file_tags[csv_path_dict.get(csv_name)] = new_csv_tags
        return new_csv_file_tags

    def write_new_csv(self):
        """写新的csv文件"""
        new_csv_file_tags = self.compare_pms_to_csv()
        for csv_file in new_csv_file_tags:
            new_csv_tags = new_csv_file_tags.get(csv_file)
            with open(csv_file, "w+", encoding="utf-8") as f:
                for tags in new_csv_tags:
                    f.write(",".join(tags) + "\n")
            logger.info(f"同步完成: {csv_file}")


if __name__ == "__main__":
    Pms2Csv().write_new_csv()



#!/usr/bin/env python3
# _*_ coding:utf-8 _*_

# SPDX-FileCopyrightText: 2023 UnionTech Software Technology Co., Ltd.

# SPDX-License-Identifier: GPL-2.0-only
# pylint: disable=C0114
import os
from enum import Enum
from enum import unique
from setting import conf


# pylint: disable=C0103


@unique
class SubCmd(Enum):
    """SubCmd"""
    run = "run"
    remote = "remote"
    pmsctl = "pmsctl"
    csvctl = "csvctl"
    startapp = "startapp"


@unique
class Args(Enum):
    """Args"""
    app_name = "app_name"
    keywords = "keywords"
    tags = "tags"
    reruns = "reruns"
    record_failed_case = "record_failed_case"
    clean = "clean"
    report_formats = "report_formats"
    max_fail = "max_fail"
    log_level = "log_level"
    timeout = "timeout"
    debug = "debug"
    noskip = "noskip"
    ifixed = "ifixed"
    send_pms = "send_pms"
    task_id = "task_id"
    trigger = "trigger"
    resolution = "resolution"
    case_file = "case_file"
    branch = "branch"
    deb_path = "deb_path"
    pms_user = "pms_user"
    pms_password = "pms_password"
    suite_id = "suite_id"
    pms_info_file = "pms_info_file"
    top = "top"
    lastfailed = "lastfailed"
    duringfail = "duringfail"
    repeat = "repeat"
    project_name = "project_name"
    build_location = "build_location"
    line = "line"
    clients = "clients"
    send_code = "send_code"
    build_env = "build_env"
    client_password = "client_password"
    parallel = "parallel"
    autostart = "autostart"
    pyid2csv = "pyid2csv"
    export_csv_file = "export_csv_file"
    pms2csv = "pms2csv"
    pms_link_csv = "pms_link_csv"
    send2task = "send2task"


def  transform_app_name(app_name):
    """转换 app_name"""
    if not app_name:
        return None
    if "-" in app_name:
        raise ValueError(f"{app_name} 中存在 ['-'] 符号")
    if app_name.startswith("apps/"):
        app_name = app_name.replace("apps/", "").strip("/")
    for dir_name in os.listdir(conf.APPS_PATH):
        if dir_name == app_name:
            return app_name
    raise NotADirectoryError(f"{app_name} 目录不存在")



#!/usr/bin/env python3
# _*_ coding:utf-8 _*_

# SPDX-FileCopyrightText: 2023 UnionTech Software Technology Co., Ltd.

# SPDX-License-Identifier: GPL-2.0-only
# pylint: disable=C0114
import os
from copy import deepcopy

from setting.globalconfig import GlobalConfig
from src  import logger


class FileCtl:
    """
    Operations on system files and directories.
    Realize the addition, deletion, modification
    and query of documents.
    """
    # pylint: disable=too-many-arguments,too-many-branches,too-many-nested-blocks
    __author__ = "Mikigo <huangmingqiang@uniontech.com>, Litao <litaoa@uniontech.com>"

    @staticmethod
    def creat_files(path: str, file_name: str = "", file_type: str = "dir"):
        """
         创建文件或文件夹
        :param path: 用户下目录下的路径
        :param file_name: 文件名，默认为空
        :param file_type: 文件类型，默认 dir，文件夹， file 文件
        """
        parent_dir = f"/home/{GlobalConfig.USERNAME}/{path}"
        if file_type in ("dir", "file"):
            if file_type == "dir":
                os.system(f"mkdir -p {parent_dir}/{file_name}")
            if file_type == "file":
                if os.path.exists(parent_dir):
                    os.system(f"touch {parent_dir}/{file_name}")
                else:
                    logger.error(f"{parent_dir} is not exists!")
        else:
            logger.error(f"{file_type} is not dir or file!")

    @staticmethod
    def delete_files(path: str, ignores: tuple = (), includes: tuple = ()):
        """
         删除文件或文件夹
        :param path: 用户目录下的路径
        :param ignores: 忽略的文件列表
        :param includes: 删除的文件列表
        """
        abs_file_path = f"/home/{GlobalConfig.USERNAME}/{path}"
        if os.path.isdir(abs_file_path):
            if not isinstance(ignores, tuple):
                logger.error("ignores Parameter is not a tuple")
                raise ValueError
            if not isinstance(includes, tuple):
                logger.error("includes Parameter is not a tuple")
                raise ValueError
            file_list = os.listdir(abs_file_path)
            if file_list:
                delete_list = deepcopy(file_list)
                if ignores and not includes:
                    for file in file_list:
                        for ignore in ignores:
                            if ignore in file:
                                delete_list.remove(file)
                    if delete_list:
                        for i in delete_list:
                            os.system(
                                f"echo '{GlobalConfig.PASSWORD}' | "
                                f"sudo -S rm -rf '{abs_file_path}/{i}'"
                            )
                elif not ignores and includes:
                    for file in file_list:
                        for include in includes:
                            if include in file:
                                os.system(
                                    f"echo '{GlobalConfig.PASSWORD}' | "
                                    f"sudo -S rm -rf '{abs_file_path}/{file}'"
                                )
                elif not ignores and not includes:
                    os.system(
                        f"echo '{GlobalConfig.PASSWORD}' | "
                        f"sudo -S rm -rf {abs_file_path}/*"
                    )
                else:
                    logger.info(
                        "This deletion mode is not supported for the time being!"
                    )
            # else:
            #     logger.info(f"There are no files in the directory <{path}!>")
        else:
            logger.error(f"{abs_file_path} is not exsits !")

    @staticmethod
    def rename_files(path: str, old_name: str, new_name: str):
        """
         重命名文件
        :param path: 用户目录下的路径
        :param old_name: 旧名字
        :param new_name: 新名字
        :return:
        """
        parent_dir = f"/home/{GlobalConfig.USERNAME}/{path}"
        old_abs_path = f"{parent_dir}/{old_name}"
        new_abs_path = f"{parent_dir}/{new_name}"
        os.system(f"mv {old_abs_path} {new_abs_path}")

    @staticmethod
    def move_files(
        path: str, file_name: str, new_path: str = None, new_file_name: str = None
    ):
        """
         移动文件
        :param path: 旧路径
        :param file_name: 旧文件
        :param new_path: 新路径
        :param new_file_name: 新文件
        :return:
        """
        parent_dir = f"/home/{GlobalConfig.USERNAME}/{path}"
        new_parent_dir = f"/home/{GlobalConfig.USERNAME}/{new_path}"
        old_abs_path = f"{parent_dir}/{file_name}"
        if os.path.exists(parent_dir):
            if not new_path:
                if new_file_name:
                    new_abs_path = f"{parent_dir}/{new_file_name}"
                else:
                    new_abs_path = old_abs_path
            else:
                if new_file_name:
                    new_abs_path = f"{new_parent_dir}/{new_file_name}"
                else:
                    new_abs_path = f"{new_parent_dir}/{file_name}"
            if os.path.exists(new_parent_dir):
                os.system(f"mv {old_abs_path} {new_abs_path}")
            else:
                raise FileNotFoundError
        else:
            raise FileNotFoundError

    @classmethod
    def find_files(
        cls,
        path: str,
        startwith: str = None,
        include: str = None,
        endwith: str = None,
        not_includes: tuple = (),
        recursive: bool = False,
        abs_path: bool = False,
    ) -> list:
        """
         查找文件
        :param path: 查找路径
        :param startwith: 以 xx 开头
        :param include: 包含的文件
        :param endwith: 以 xx 结尾
        :param not_includes: 不包含的文件夹
        :param recursive: 是否查找子目录
        :param abs_path: 是否返回绝对路径列表。
        :return: 查找到的文件列表
        """
        parent_dir = f"/home/{GlobalConfig.USERNAME}/{path}"
        if os.path.exists(parent_dir):
            files_list = []
            if recursive:
                for root, _, files in os.walk(parent_dir):
                    for file in files:
                        cls.__check_not_includes(
                            not_includes,
                            endwith,
                            file,
                            files_list,
                            include,
                            startwith,
                            abs_path,
                            root,
                        )
            else:
                for file in os.listdir(parent_dir):
                    cls.__check_not_includes(
                        not_includes,
                        endwith,
                        file,
                        files_list,
                        include,
                        startwith,
                        abs_path,
                        parent_dir,
                    )
            if not files_list:
                logger.info(
                    f"查找模式：\nstartwith:{startwith}\ninclude:{include}\n"
                    f"endwith:{endwith}\nrecursive:{recursive}\n"
                    f"{parent_dir}目录下未找到该查找模式的文件～"
                )
            return files_list
        logger.error(f"{parent_dir} is not exists!")
        return []


    @classmethod
    def __check_not_includes(
        cls, not_includes, endwith, file, files_list, include, startwith, abs_path, root
    ):
        """Filter criteria for function 'find_files' parameter 'notincludes'"""
        if not_includes:
            for not_include in not_includes:
                if not_include in file:
                    continue
                cls.__check_startwith(
                    endwith, file, files_list, include, startwith, abs_path, root
                )
        else:
            cls.__check_startwith(
                endwith, file, files_list, include, startwith, abs_path, root
            )

    @classmethod
    def __check_startwith(
        cls, endwith, file, files_list, include, startwith, abs_path, root
    ):
        """Filter criteria for function 'find_files' parameter 'startwith'"""
        if not startwith:
            cls.__check_endwith(endwith, file, files_list, include, abs_path, root)
        else:
            if file.startswith(startwith):
                cls.__check_endwith(endwith, file, files_list, include, abs_path, root)

    @classmethod
    def __check_endwith(cls, endwith, file, files_list, include, abs_path, root):
        """Filter criteria for function 'find_files' parameter 'endwith'"""
        if not endwith:
            cls.__check_include(file, files_list, include, abs_path, root)
        else:
            if file.endswith(endwith):
                cls.__check_include(file, files_list, include, abs_path, root)

    @classmethod
    def __check_include(cls, file, files_list, include, abs_path, root):
        """Filter criteria for function 'find_files' parameter 'include'"""
        if not include:
            cls.__check_abspath(file, files_list, abs_path, root)
        else:
            if include in file:
                cls.__check_abspath(file, files_list, abs_path, root)

    @classmethod
    def __check_abspath(cls, file, files_list, abs_path, root):
        """Filter criteria for function 'find_files' parameter 'abs_path'"""
        if not abs_path:
            files_list.append(file)
        else:
            files_list.append(f"{root}/{file}")

    @staticmethod
    def file_exists(*files, abspath=False, _try=True):
        """
         查找文件，若有不存在的文件则返回None货抛出异常
        :param files: 文件列表
        :param abspath: 是否绝对路径
        :param _try: 是否抛出异常
        :return: 文件列表
        """
        if not abspath:
            abspath = f"/home/{GlobalConfig.USERNAME}"
        for file in files:
            if not os.path.exists(f"{abspath}/{file}"):
                if _try:
                    raise FileNotFoundError(file)
                return None
        return files

    @staticmethod
    def file_name_and_format(position, _format=None):
        """
         获取文件的名称和格式, 返回名称和格式组成的tuple
        :param position: 路径
        :param _format: 以 xx 结尾
        :return: 名称和格式
        """
        files = FileCtl.find_files(path=position, endwith=_format)
        for file in files:
            name_pic = file.split(".")[0]
            format_pic = file.split(".")[1]
            if "test_" in name_pic:
                return name_pic, format_pic
            return None, None
        return None, None


if __name__ == "__main__":
    FileCtl.delete_files("Desktop", includes=("新建",))
